/* POSTOUT -- Uses the dBase IV DLD file to enable you to output a standard
	ASCII text file to a PostScript printer without modifying it. Only prints
	7 bit ASCII set. Higher ASCII characters are converted to *'s. Tabs are
	converted to spaces as specified by command line parameters.

	Created by Martin Leon, Ashton Tate Software Support
    Version 1.1a */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#define MAXPATH 256

FILE *infile;
FILE *outfile;
FILE *dldfile;

/* ANSI C function prototypes */
int get_slash_arg( char switch_letter, char *argv[] );
char *get_eq_arg( char prefix, char *argv[] );

/* Names of PostScript macros created in the DLD file */
char crlf[] = "CRLF\n";
char show[] = "show\n";
char ffeed[] = "FF \n";
char font[3][7] = {"1FONT\n","2FONT\n","3FONT\n"};

/* Other variables */
char tab[11] = "          "; /* tab set to max of 10 spaces */
char homedir[MAXPATH];
char temp[MAXPATH];
char *ptr;
int linecount = 0;
int max_lines = 0;
float point_size = 0.0f;
int c = '\0';
int x = 0;
int tabspace;
int fontnumber = 0;
int in_parens = 0;

int main( int argc, char *argv[] )
{
	/* Check for at least 2 command line arguments */
	if( argc < 3 ) {
		printf( "\nPOSTOUT.EXE v1.1a by Martin R. Len\n\n"\
         "Input and output file name required. E.G.: POSTOUT IN.TXT OUT.TXT\n\n"\
			"Output file can be any output device or file name.\n\n"\
			"Optional switches (can be used in any order/combination):\n"\
			"   /C for compressed print\n"\
			"   /L for landscape printing\n"\
			"   T=<number between 3 & 10> for tab stop spacing <default is 3>\n"\
			"   F=<number between 1 & 3> for font number\n"\
			"      1 is Courier, fixed spacing <default font>\n"\
			"      2 is Helvetica, proportional spacing\n"\
			"      3 is Times, proportional spacing\n"\
			"   L=<number between 45 & 78> for page length <default is 63; 45 in landscape>\n"\
			"   P=<number between 1.00 & 99.99> for point size <default is 12>\n" );
		return 1;
	}

	/* Check for tab setting switch and adjust tab spacing if needed */
	tabspace = atoi( get_eq_arg( 'T', argv ) );
	if( (tabspace < 3 || tabspace > 10) && tabspace != 0 ) {
		printf( "\nTab setting must between 3 and 10\n" );
		return 1;
	}
	if( tabspace != 0 ) {
		ptr = tab;
		ptr += strlen( tab );
		while( (int)strlen( tab ) > tabspace ) {
			ptr--;
			*ptr = '\0';
		}
	}
	else
		strcpy( tab ,"   " );

	/* Check for font setting switch and set fontnumber */
	fontnumber = atoi( get_eq_arg( 'F', argv ) );
	if( fontnumber > 3 ) {
		printf( "\nFont number must be between 1 and 3\n" );
		return 1;
	}

	/* Check for point size setting at command line */
	point_size = (float)atof( get_eq_arg( 'P', argv ) );
	if( (point_size < 1.00f || point_size > 99.99f) && point_size != 0.0f ) {
		printf( "\nPoint size must be between 1.00 and 99.99\n" );
		return 1;
	}

	/* Check for page length setting and adjust if needed */
	max_lines = atoi( get_eq_arg( 'L', argv ) );
	if( max_lines < 10 || max_lines > 128 )
		if( max_lines == 0 )
			if( get_slash_arg( 'L', argv ) )
				max_lines = 45;
			else
				max_lines = 63;
	else {
		printf( "\nPage length must be between 10 and 128\n" );
		return 1;
	}

	/* Make sure input file can be openned */
	if( ( infile = fopen( argv[1], "rt" ) ) == NULL ) {
		printf( "\nCannot open %s for input.\n", argv[1] );
		fcloseall();
		return 1;
	}
	/* Make sure output file can be openned */
	if( ( outfile = fopen( argv[2], "wt" ) ) == NULL ) {
		printf( "\nCannot open %s for output.\n", argv[2] );
		fcloseall();
		return 1;
	}
	/* Determine directory POSTOUT was called from and add DLD file name to it.*/
	/* This will only work for DOS version 3.0 and higher */
	strcpy( homedir, argv[0] );
	ptr = homedir;
	ptr += (strlen( homedir ) - 1 );
	while( *ptr != '\\' )
		ptr--;
   strcpy( ++ptr, "POSTSCRI.DLD" );

	/* Make sure dld file can be openned from same dir as POSTOUT.EXE */
   if( ( dldfile = fopen( homedir, "rt" ) ) == NULL ) {
		printf( "\nCannot open %s\n"\
			"DLD file must be in same directory as POSTOUT.EXE\n", homedir );
		fcloseall();
		return 1;
	}

	/* copy dld file to output */
	printf( "\nWriting Output\r" );
	while( (c = fgetc( dldfile )) != EOF )
		fputc( c, outfile );

	/* If you are not using an HP III with a PostScript cartridge,
		add comment delimiters to next line */
	fputs( "/tlxoff 18 def \n", outfile );

	/* Insert call in output to DLD Init macro to initialize printjob */
	fputs( "Init\n", outfile );

	/* If /L command line argument was used, set to landscape printing */
	if( get_slash_arg( 'L', argv ) )
		fputs( "LAND\n", outfile );

	/* If /C command line argument was used, set to compressed printing
		otherwise set to normal printing */
   if( get_slash_arg( 'C', argv ) )
   	fputs( "Cmp+\n", outfile );
	else
   	fputs( "Norm\n", outfile );

   /* send font selected or set to default of first font */
	if( fontnumber != 0 )
		fputs( font[ fontnumber - 1 ], outfile );
	else
		fputs( font[0], outfile );

	/* adjust point size if requested from command line */
	if( point_size != 0 )
		fprintf( outfile, "/CurPoint %5.2f def\nNewFont\n", point_size );

	/* Format text for PostScript and send to output */
	fputc( '(', outfile );
	in_parens = 1;
	while( ( c = fgetc( infile ) ) != EOF ) {
		/* Translate form feed characters */
		if( c == '\x0C' ) {
			if( in_parens ) {
				fputc( ')', outfile );
				in_parens = 0;
				fputs( show, outfile );
			}
			fputs( ffeed, outfile );
			linecount = 0;
			fputc( '(', outfile );
			in_parens = 1;
			continue;
		}
		/* Replace carraige returns with CRLF macro in DLD */
		if( c == '\n' ) {
			/* If at last line on page, call FF macro in DLD */
			if( ++linecount == max_lines ) {
				if( in_parens ) {
            	fputc( ')', outfile );
					in_parens = 0;
				}
				fputs( show, outfile );
				fputs( ffeed, outfile );
				fputc( '(', outfile );
				in_parens = 1;
				linecount = 0;
				continue;
			}
			/* Close parens on SHOW macro in DLD and add CRLF macro */
			if( in_parens ) {
        		fputc( ')', outfile );
				in_parens = 0;
			}
			fputs( show, outfile );
			fputs( crlf, outfile );
         fputc( '(', outfile );
			in_parens = 1;
			continue;
		}
		/* Replace some control character and all ASCII characters
			with values greater than 128 in input with an asterisk */
		if( c > '\x7F' || c < '\t' || (c < ' ' && c > '\n') ) {
			if( c == '\t' )
				fputs( tab, outfile );
			else
            fputc( '*', outfile );
			continue;
		}
		/* Replace tab character in input with number of spaces determined by
			command line argument */
		if( c == '\t' ) {
			fputs( tab, outfile );
			continue;
		}
		/* Replace backslashes and parens in input to \\ and \( and \) */
      if( c == '\\'  || c == '(' || c == ')' )
			fputc( '\\', outfile );

		fputc( c, outfile );
	}

	/* End of file reached -- close parens for SHOW macro */
	fputc( ')', outfile );
	fputs( show, outfile );
	/* Send FF for macro in DLD */
	fputs( ffeed, outfile );
	/* Send EOT character to indicate end of job */
	fputc( '\x04', outfile );
	/* Close files */
	fcloseall();
	printf( "File Written to %s\n", argv[2] );
	return 0;
}


/* Scans all command line arguments for one that begins with a specific
	letter and is followed by an = sign. Returns pointer to first character
	after the equal sign */

char *get_eq_arg( char prefix, char *argv[] ) /* ANSI C function header */
{
	char *temptr = *argv;
	while( *argv )
	{
      int x = 0;
		char *ptr = strcpy( temp, *argv );
		temptr = ptr;

		if( toupper( *ptr++ ) == prefix && *ptr++ == '=' )
      	return ptr;
      argv++;
	}
  	temptr = (char *)argv;
   return temptr;
}

/* Scans all command line arguments for one that begins with a slash and is
	followed by a specific letter. Returns 1 if switch is found in arguments */

int get_slash_arg( char switch_letter, char *argv[] ) /* ANSI C function header */
{
	char *temptr = *argv;
	while( *argv )
	{
      int x = 0;
		char *ptr = strcpy( temp, *argv );
		temptr = ptr;

		if( *ptr == '/' &&  toupper( *(++ptr) ) == switch_letter )
      	return 1;
      argv++;
	}
   return 0;
}
